/* 
 * IpoptMexInterface.h -- include file for IPOPT mex interface 
 *
 * $Id$
 * $Source$
 */

#include "mex.h"


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#ifdef __UNIX__
#include <sys/time.h>
#include <unistd.h>
#endif

#include "Ipopt_Interface.h"

extern int errno;

#ifndef false
#define false (0)
#endif
#ifndef true 
#define true (1)
#endif

#define MYSELF   "IPOPT"
#define INF      1E10;
#define BUFLEN    128

#define MIN(a,b)        ((a) <= (b) ? (a) : (b))
#define MAX(a,b)        ((a) >= (b) ? (a) : (b))

#define MSIZE(p)        ( mxGetM(p) * mxGetN(p) )
#define IS_SCALAR(p)    ( MSIZE(p) == 1 )
#define IS_VECTOR(p)    ( MIN( mxGetM(p), mxGetN(p) ) == 1 )
#define N_VECTOR(p,n)   ( (IS_VECTOR(p)) && (MSIZE(p) == (n)) )


void           flush();
void           MyInfo(char *msg, int msg_lvl);
void           MyDebug(char *msg, int msg_lvl);
void           MyIpoptError(char *msg);
long double    GetTimeStamp(void);

fint*   CreateFintVector(int Capacity);
double* CreateDoubleVector(int Capacity);
double* insertZeroAt(double* vector,int newlength, int position);
double* AddToVector(int howMany,double *Vector, double *Values);
double  RandomNumber(double min, double max);
void*   setRandomValidPoint(double *_point,int _n);

void    setPRHS(int _nrhs, int _rowsrhs, int _colsrhs,
		double *_x,double *_lambda, int _n,
		int _NoOfCons, int _method,
		mxArray **L_prhs, int _task);

int    destroyPRHS(mxArray **prhs, int _method, 
		   int _nrhs, int _NoOfCons, int task); 

int    InitSparseMatrixPattern(int actuallength, int colslhs, int _n,
			       mxArray **plhs, fint *rowindex,
			       fint *colindex, int method);
			      

int    InitFullMatrixPattern(int colslhs, int rowslhs, fint *rowindex,
			     fint *colindex, int _n, int resultlength,  
			     int method);

int    FillInExtraZeros(int actuallength, int maxlength, int colslhs,
			mxArray **plhs, double *result, char *now_evalfname);

int    AnalyzeSparseMatrix(int _rows, int _cols, int _n, int _nrhs,
			   char *_name, int _method, int _reassurance,
			   int _m, bool CalcJacobian, bool CalcHessian);
			   

int    CallMatlabSparse(int _nzmax, int _n, int nrhs, int nlhs,
			real *_x, real *result, int rowslhs,
			int colslhs, int rowsrhs, int colsrhs, 
			int resultlength, char *_matlabFuncName, 
			real *_lambda, fint *rowindex, 
			fint *colindex, int maxlength,
			int task, int method);  

int    CallMatlabFull(int _nzmax, int _n, int nrhs, int nlhs,
		      real *_x, real *result, int rowslhs, int colslhs,
		      int rowsrhs, int colsrhs, int resultlength, 
		      char *_matlabFuncName, real *_lambda,
		      fint *rowindex, fint *colindex,
		      int task, int method);  

int     AnalyzeMatrix(int _n, int _m, int _rows, int _cols, 
		      int _nrhs, int _reassurance, char *_name,
		      bool *_isSparse, int matrix);

int     eval_F(fint _n, real *_x, real *_f, void* _userData);

int     eval_G(fint _n, real *_x, real *_g, void *_userData);

int     eval_C(fint _n, real *_x, fint _m, real *_c, void *_userData); 

int     eval_A(fint _task, fint _n, real *_x, fint *_nz,
	       real *_A, fint *_Acon, fint *_Avar, void *_userData); 

int     eval_H(fint _task, fint _n, real *_x, fint _m,
	       real *_lambda, fint *_nnzh, real *_hess,
	       fint *_irnh, fint *_icnh, void *_userData);
