//**********************************************************************
//* $Id$ 
//* $Source$
//*
//* Part of Scialb Mex interface for IPOPT
//*
//* This code is published under the Common Public License
//* http://www.opensource.org/licenses/cpl.php
//*
//* Copyright (C) 2004; All Rights Reserved
//*
//* Claas Michalik <Michalik<at>LPT<dot>RWTH-Aachen<dot>de>
//* Process Systems Engineering Institute, RWTH Aachen
//*
//* Steinar Hauan <hauan@cmu.edu>
//* Dept of ChemE, Carnegie Mellon University
//*
//* Modifications to Scilab Interface made by
//* Edson Cordeiro do Valle <edsoncv@vrtech.com.br>
//* Departamento de Desenvolvimento e Tecnologia, VRTech Tecnologias Industriais LTDA.
//* www.vrtech.com.br
//**********************************************************************

// This is an example for the usage of IPOPT (using the Scilab interface)
// It implements problem 71 from the Hock-Schittkowsky test suite:
// 
//  min   x1*x4*(x1 + x2 + x3)  +  x3
//     s.t.  x1*x2*x3*x4                    -  25  >=  0
//           x1**2 + x2**2 + x3**2 + x4**2  -  40  =  0
//           1 <=  x1,x2,x3,x4  <= 5
//
//  Since the interface don't support inequality constraints (but later will!!!), the problem
//  is rewritten as:
//     min   x1*x4*(x1 + x2 + x3)  +  x3
//     s.t.  x1*x2*x3*x4  -  x5             -  25  =  0
//           x1**2 + x2**2 + x3**2 + x4**2  -  40  =  0
//           1 <=  x1,x2,x3,x4  <= 5
//           0 <=  x5
// 
//    Starting point:
//    x = (1, 5, 5, 1, -24)
// 
//    Optimal solution:
//    x = (9.99999990e-01, 4.74299964e+00, 3.82114998e+00, 1.37940831e+00, 0)
getd();
StartPoint = [1 5 5 1 -24];
NoEquCons  = 2; // number of equality constratins
i_lb   = [1,2,3,4,5]; 
val_lb = [1,1,1,1,0];
i_ub   = [1,2,3,4];
val_ub = [5;5;5;5];
Par = [];

//Evaluate the jacobians/gradients analiticaly
[Xfinal,ObjFinal,mlb,mub,lambda,info,XInfo,ObjInfo] = Ipopt('eval_f','eval_c','eval_g','eval_a','eval_h', NoEquCons,StartPoint, i_lb, val_lb, i_ub,val_ub,Par,'print',1)
//Evaluate the jacobians/gradients by finite differences

[Xfinal,ObjFinal,mlb,mub,lambda,info,XInfo,ObjInfo] = Ipopt('eval_f','eval_c','fd','fd','fd', NoEquCons,StartPoint, i_lb, val_lb, i_ub,val_ub,Par,'print',1)

//You can use the internal option of ipopt "IQUASI" to evaluate the Hessian's Jacobian 
// by finite differences (it is much faster) 
[Xfinal,ObjFinal,mlb,mub,lambda,info,XInfo,ObjInfo] = Ipopt('eval_f','eval_c','fd','fd','fd', NoEquCons,StartPoint, i_lb, val_lb, i_ub,val_ub,Par,'print',1,'IQUASI',6)

