# VRTech additions to www.scons.org.
#
# This file should not be used directly, use the SConstruct which comes with
# the distribution instead.
#
# This file enables to reswrap images with:
# env.Reswrap('icons', Split('colors.gif information.gif question.gif'))
#
# And build FOX applications with:
# env.Program('foxprogram', Split('main.cpp HelWindow.cpp icons.cpp'))
#
# Copyright (c) 2004-2005 VRTech Tecnologias Industriais Ltda www.vrtech.com.br
# Author: Rafael de Pelegrini Soares
#

Import('env')
import os
import os.path
import sys
import string
import SCons.Util

#============================================================================
# Baisc definitions for MS Visual C++
#
try:
	if env['msvc']:
		env['platform'] = 'win32'
		env['host'] = 'win32'
		env['configuration'] = 'release'
		env['LAUNCH_DIR'] = '.' # os.path.abs('.')
		env['LIBDIR'] = '#lib/msvc'
		env['BINDIR'] = '#bin/msvc'
except:
	env['msvc'] = 0

#============================================================================
# Cross compiling stuff
#
target = env['platform']
host = env['host']
if not host == target:
        cross = False
        if target == 'win32':
                # cross compiling with mingw get the cross-tools at
                # http://www.libsdl.org/extras/win32/cross/ and install
                # it under /usr/local/cross-tools
                key_program = 'i386-mingw32msvc-gcc'
                mingw = env.WhereIs(key_program)
                if mingw:
                        env.Tool('mingw')
                        dir = os.path.dirname(mingw)
                        # The mingw bin directory must be added to the path:
                        path = env['ENV'].get('PATH', [])
                        if not path: path = []
                        if SCons.Util.is_String(path): path = string.split(path, os.pathsep)
                        env['ENV']['PATH'] = string.join([dir] + path, os.pathsep)
                        # a few things differ from std mingw:
                        env['F77'] = 'g77'
                        tools = ['CC', 'CXX', 'LINK', 'AS', 'RC', 'AR', 'RANLIB', 'F77']
                        for i in tools: env[i] = 'i386-mingw32msvc-' + env[i]
                        env['PROGSUFFIX'] = '.exe'
                        env['SHLIBSUFFIX'] = '.dll'
                        env['SHLIBPREFIX'] = ''
                        cross = True
                        # FIXME: add other cross compiling tools
        if not cross:
                print '*** error: cross compilation for given target unavailable'
                Exit(1)
if target == 'win32' and not env['msvc']:
    env.Append(LINKFLAGS = ['-Wl,--enable-runtime-pseudo-reloc', '-Wl,--enable-auto-import'])
    if env['configuration'] == 'release':
	    env.Append(LINKFLAGS = ['-s'])

#====================================================================
# Check for FOX function.
# With the function defined below you can check for FOX usability
# with the following commands:
#
# conf = env.Configure(custom_tests = { 'CheckFOX' : env['CheckFOX'] } )
# if not conf.CheckFOX():
#	print '*** error: no usable FOX library found!'
#	Exit(1)
# env = conf.Finish()
#
# Instead of exit, the user can conditionally build the application
# or not.
#
def CheckFOX(context):
    context.Message( 'Checking FOX usability... ' )
  
    if host == 'win32' and not env['msvc']:
         foxinclude = '/mingw/include/fox-1.4'
         context.env.Append(LIBS = ['FOX-1.4', 'comctl32', 'wsock32', 'winspool', 'mpr','m', 'opengl32', 'glu32', 'gdi32'])
         context.env.Append(CPPPATH = foxinclude)
         context.env.Tool('mingw')
         ret = True
         context.Result( ret )
         return ret
    if env['msvc']:
         context.env.Append(LIBS = ['FOX-1.4'])
         context.env.Append(LIBS = Split('kernel32 user32 gdi32 winspool comdlg32 advapi32 shell32 ole32 oleaut32 uuid'))
         context.env.Append(LINKFLAGS = ['/machine:I386'])
         context.env.Append(LINKFLAGS = ['/subsystem:windows', '/entry:\"mainCRTStartup\"'])
         ret = True
         context.Result( ret )
         return ret

    # Another hosts
    foxconfig = context.env.WhereIs('fox-config', pathext = '')
    if not foxconfig:
	    print '*** error: fox-config not found'
	    ret = False
    else:
        context.env.Append(LINKFLAGS = '-mwindows')
        context.env.ParseConfig(foxconfig + ' --cflags --libs')
        ret = context.TryLink("""
                #include <fx.h>
                int main(int argc, char **argv) {
                FXApp app("name", "vendor");
                app.init(argc, argv);
                app.create();
                return 0;
                }
               """, '.cpp')
    context.Result( ret )
    return ret
env['CheckFOX'] = CheckFOX

#===========================================================================
# ResWrap builder
# Fox-toolkit reswrap to embed images in the application.
#
# Use it as follows:
# env.Reswrap(target = ['images.cpp', 'images.h'], source = ['icon1.gif', 'figure1.bmp']
# This will generate images.cpp and images.h with the given (source) figures reswrapped.
#
# If you pass only one target, lets say target = 'icons', then icons.cpp
# and icons.h will be generated.
#
# If no target is given as follows:
# env.Reswrap(['icon.gif', 'icon2.gif'])
# then icon.cpp and icon.h are generated with given (source) figures reswrapped.
#
env['RESWRAP'] = 'reswrap'
def reswrapemitter(target, source, env):
	if len(target) == 1:
		target = [ str(target[0]) + '.cpp', str(target[0]) + '.h'  ]
	elif not len(target) == 2:
		print '*** error: Reswrap needs two targets'
		Exit(1)
	if not env.WhereIs(env['RESWRAP']):
		print '*** error: reswrap not found'
		Exit(1)
	return (target, source)
reswrapbuilder = Builder(
	emitter = reswrapemitter,
	action = ['$RESWRAP -e -o ${TARGETS[0]} ${SOURCES}',
	'$RESWRAP -i -o ${TARGETS[1]} $SOURCES'])
env.Append(BUILDERS = {'Reswrap' : reswrapbuilder})



#==================================================================
# Mode defines: debug, release, flags, etc
#
if env['configuration'] == 'debug' :
	env.Append(CPPDEFINES= ['DEBUG', '_DEBUG'] )
	env.Append(CCFLAGS='-g')
	env.Append(F77FLAGS='-g')
	env.Append(SHF77FLAGS='-g')
else :
	env.Append(CPPDEFINES= ['NDEBUG'] )
	if env['msvc']:
		env.Append(CCFLAGS='/O2')
	else:
		env.Append(CCFLAGS='-O2')
		env.Append(F77FLAGS='-O3')

if env['msvc']:
	env.Append(CPPDEFINES = ['WIN32'])
	env.Append(LIBPATH = env['LIBDIR'])
	env.Append(CPPFLAGS = ['/MT'])
	env.Append(CPPFLAGS = ['/W3'])
	env.Append(LINKFLAGS = ['/NODEFAULTLIB:LIBC'])
	env.Append(CCFLAGS = ['/EHsc'])
else:
	env.Append(CCFLAGS = ['-Wall'])
	env.Append(CCFLAGS = ['-fexceptions'])
	env.Append(F77FLAGS = ['-fexceptions'])
	env.Append(SHF77FLAGS = ['-fexceptions'])

if not target == 'win32':
	#env.Append(LINKFLAGS = '--Wl,--no-undefined')
	env.Append(SHF77FLAGS = ['-fPIC'])

#==============================================================================
# IDL builder
# Compile an CORBA idl file using omniidl(omniORB)
# example:
# env.Idl(source = 'gco.idl')
# will generate gco.cpp and gco.hpp
#
env['OMNIIDL'] = 'omniidl'
def omniidlemitter(target, source, env):
	if len(source) != 1:
		print '*** error: omniidl builder expects exactly one source file'
		Exit(1)
	if not env.WhereIs(env['OMNIIDL']):
		print '*** error: omniidl not found'
		Exit(1)
	s = str(source[0])
	s_filebase, s_ext = os.path.splitext(s)
	targets = [s_filebase + i for i in ['.hpp', '.cpp']]
	return (targets, source)
omniidlcom = '$OMNIIDL -C ${SOURCE.dir} -T -bcxx -Wbh=.hpp -Wbs=SK.cpp -Wbtp -Wbexample $SOURCE'
env['OMNIIDLCOM'] = omniidlcom
idlbuilder = Builder(action = omniidlcom, emitter = omniidlemitter)
env.Append(BUILDERS = {'Idl' : idlbuilder})

#=============================================================================
# ANTLR builder
# Compiles an antlr grammar file.
# This builder requires java and antlr.jar in your CLASSPATH
env['ANTLRJAVA'] = 'java'
if not host == 'win32':
	env['ANTLRCLASSPATH'] = '/home/rafael/devel/antlr'
else:
	env['ANTLRCLASSPATH'] = '../../antlr'
antlrcomm = '$ANTLRJAVA -cp ${ANTLRCLASSPATH}/antlr.jar antlr.Tool -o ${SOURCE.dir} $SOURCE'
def antlrEmitter(target, source, env):
	if len(source) != 1:
		print '*** error: antlr builder expects exactly one source file'
		Exit(1)
	if not env.WhereIs(env['ANTLRJAVA']):
		print '*** error: java not found'
		Exit(1)
#	print env['ANTLRCLASSPATH']+'/antlr.jar'
	if (not host == 'win32') and not os.path.isfile(env['ANTLRCLASSPATH']+'/antlr.jar'):
		print '*** error: antlr.jar not found'
		Exit(1)
	s = str(source[0])
	s_filebase, s_ext = os.path.splitext(s)
	print s_filebase
	print env['BUILD_DIR']
	s_filebase = os.path.join(s_filebase, env['BUILD_DIR'])
	targets = [s_filebase + i for i in
		['Lexer.hpp', 'Parser.hpp', 'Lexer.cpp', 'Parser.cpp']]
	return (targets, source)
antlrbuilder = Builder(action = antlrcomm,
		src_suffix = '.g', emitter = antlrEmitter)
env.Append(BUILDERS = {'Antlr' : antlrbuilder})


#==============================================================================
# list files recursivelly given one directory
#
#
def listfiles(dir, undesired):
	list = []
	# all files on the directory
	files=os.listdir(dir)
	# filtering accordingly the patterns
	for file in files :
	  desired = 1
	  for i in undesired :
	    if file.endswith(i):
	      desired = 0
	      break;
	  if desired :
	    if os.path.isdir(os.path.join(dir,file)) :
	      list.append(listfiles(os.path.join(dir,file), undesired))
	    else :
	      list.append(os.path.join(dir,file))
	return list



#==============================================================================
# The zip default target
#
undesired = ['.obj', '.o', '.build', 'bin', 'lib', '.tmp', '.bak',
'~', '.sconsign','.idb', '.pdb', '.pch', '.opt', '.ncb', '.pch', '.ilk', '.suo',
'.dep', '.zip','.tar', '.tar.gz', '.os', '.la', '.lo', '.metadata',
'.old', '.sconf_temp', '.swp']

files = listfiles(env['LAUNCH_DIR'], undesired)
name = os.path.join(os.path.join(env['LAUNCH_DIR'],'..'),os.path.basename(env['LAUNCH_DIR']))
zip = env.Zip(name, files)
env.Alias('zip', zip)

